<?php

/**
 *
 *
 * @author    Matthias
 * @created   29.07.2009 15:39:26
 * @copyright 2009
 */



class WPSCB_Helpers {

        /**
         * returns the cache directory
         */
        function getCacheDir() {
                return dirname(__FILE__) . '/cache';
        }

        /**
         * escapes some special chars for cache file
         * @param string string to escape
         */
        function prepareStringForCacheFile($string) {
                $string = str_replace(array('"', '$'), array('\"', '\$'), $string);
                return $string;
        }

        /**
         * removes all tags and interpunction chars
         * @param string text to clean up
         */
        function cleanUpText($text) {
                // remove tags
                $text = preg_replace('!<[^>]+>(.+)</[^>]+>!sU', '\1', $text);
                // remove interpunction
                $text = preg_replace('#[^\p{L}\p{N}]+#u', ' ', $text);
                // get back
                return ' ' . $text . ' ';
        }

        /**
         * checks url
         * @param string url
         */
        function isUrl($url) {
                return preg_match('#^(ht|f)tp(s)?\:\/\/[a-z0-9\-]+\.([a-z0-9\-]+\.)?[a-z]+#i', $url) ? true : false;
        }

        /**
         * checks email
         * @param string email
         */
        function isEmail($email) {
                return preg_match('|^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]{2,})+$|i', $email) ? true : false;
        }

        /**
         * is the radio checked or not?
         * @param string index of option
         * @param string value of option
         * @param array  options
         */
        function isChecked($index, $value, $options) {
                echo ($options[$index] == $value ? ' checked="checked"' : '');
        }

        /**
         * calculates whether the bot will comment or not
         * @param string quantity
         */
        function calculateChance($chance) {
                $sum = 16;
                switch ($chance) {
                        // 75%
                        case 'often':
                            $amount = str_pad('', ($sum * 0.75), 'y') . str_pad('', ($sum * 0.25), 'n');
                            break;
                        // 50%
                        default:
                        case 'medium':
                            $amount = str_pad('', ($sum * 0.5), 'y') . str_pad('', ($sum * 0.5), 'n');
                            break;
                        // 25%
                        case 'rare':
                            $amount = str_pad('', ($sum * 0.25), 'y') . str_pad('', ($sum * 0.75), 'n');
                            break;
                }
                // get the random result whether to post or not
                $rand = rand(0, $sum-1);
                $result = substr($amount, $rand, 1);
                return $result;
        }

        /**
         * removes <blockquote> from content
         * @param string content
         */
        function removeOldBlockquotes($content) {
                return trim(preg_replace('|<blockquote>.+</blockquote>|is', "", $content));
        }

        /**
         * turns first letter of a string to uppercase
         * @param string string
         */
        function camelizeFirstWordOfString($string) {
                $str = substr($string, 1);
                $first = strtoupper(substr($string, 0, 1));
                return $first . $str;
        }

        /**
         * gets all files of a directory
         * @param string path to directory
         * @param string (optional) file to exclude
         */
        function getDirList($directory, $exclude = '')  {
                $results = array();
                $handler = opendir($directory);
                while ($file = readdir($handler)) {
                        if ($file != '.' && $file != '..' && $file != $exclude) {
                                $results[$file] = $file;
                        }
                }
                closedir($handler);
                return $results;
        }

}