<?php
/*
Plugin Name: WPSuperCommentBot
Plugin URI: http://www.superlevel.de/wordpress/wpsupercommentbot
Description: This is what the Blogosphere has been waiting for! A Superbot which automatically comments on your articles and responds to your readers comments. OMFG! Just plug it in and get started. Crazy!
Author: Matthias Kunze
Version: 0.8.16
Author URI: http://www.superlevel.de
*/

if (!class_exists('WPSuperCommentBot')) {

        require 'wpscb_helpers.php';

        class WPSuperCommentBot extends WPSCB_Helpers {

                var $version = '0.8.16';

                // standard bot url
                var $url = 'http://www.superlevel.de/wordpress/wpsupercommentbot';

                // standard inc language
                var $standardLanguage = 'de_DE';

                // standard options
                var $standards = array(
                        'bot_id' => 0,
                        'bot_name' => 'random',
                        'bot_name_string' => '',
                        'bot_activity' => 'medium',
                        'bot_reaction' => 'random',
                        'bot_interaction' => 'yes',
                        'bot_url' => 'superlevel',
                        'bot_url_string' => '',
                        'bot_mail_string' => ''
                );

                // range in which the bot will comment a post
                var $minMinutesFromNow = 15; //15;
                var $maxMinutesFromNow = 240; //240;

                // range in which the bot will comment on comments
                var $minMinutesNextComment = 5; //5;
                var $maxMinutesNextComment = 90; //90;

                // max bot comments per post - depends on activity setting
                var $maxBotComments = array(
                        'often' => 8,
                        'medium' => 4,
                        'rare' => 2
                );

                /**
                 * PHP 4 *haha!* ^^
                 */
                function WPSuperCommentBot() {
                        $this->__construct();
                }

                /**
                 * init
                 */
                function __construct() {
                        $this->standards['bot_mail_string'] = get_option('admin_mail');

                        $basename = basename(dirname(__FILE__));
                        $this->pluginDir = get_option('siteurl') . '/wp-content/plugins/' . $basename;

                        register_activation_hook(__FILE__, array(&$this, 'activateWPSuperCommentBot'));
                        register_deactivation_hook(__FILE__, array(&$this, 'deactivateWPSuperCommentBot'));

                        add_filter('admin_menu', array(&$this, 'addWPSuperCommentBotAdminPage'));
                        add_filter('save_post', array(&$this, 'startWPSuperCommentBotOnPostSave'));
                        add_filter('comment_post', array(&$this, 'startWPSuperCommentBotOnCommentSave'));

                        // schedule event hooks
                        add_filter('wpscb_schedule', array(&$this, 'autoCommentOnPost'));
                        add_filter('wpscb_schedule_comment', array(&$this, 'autoCommentOnComment'));

                        load_plugin_textdomain('wpsupercommentbot', 'wp-content/plugins/' . $basename, $basename);

                        if ($this->checkCacheFolder() === false) {
                                add_filter('admin_notices', array(&$this, 'showWpscbWarning'));
                        }
                }

                /**
                 * plugin activated
                 */
                function activateWPSuperCommentBot() {
                        // standard options
                        if (get_option('WPSuperCommentBot') === false) {
                                $random_password = wp_generate_password(12, false);
                                $user_id = wp_create_user('WPSuperCommentBot', $random_password);
                                $this->standards['bot_id'] = $user_id;
                                add_option('WPSuperCommentBot', $this->standards);
                        }
                }

                /**
                 * plugin deactivated
                 */
                function deactivateWPSuperCommentBot() {
                        // remove scheduled comment bot actions
                        // wp_clear_scheduled_hook doesn't work
                        // so we do it manually
                        $cron = _get_cron_array();
                        foreach ($cron as $index => $c) {
                                if (isset($c['wpscb_schedule'])) {
                                        unset($cron[$index]['wpscb_schedule']);
                                }
                                if (isset($c['wpscb_schedule_comment'])) {
                                        unset($cron[$index]['wpscb_schedule_comment']);
                                }
                                if (count($cron[$index]) == 0) {
                                        unset($cron[$index]);
                                }
                        }
                        _set_cron_array($cron);
                }

                /**
                 * adds admin panel
                 */
                function addWPSuperCommentBotAdminPage() {
                        add_submenu_page('plugins.php', 'WPSuperCommentBot ' . __('Configuration', 'wpsupercommentbot'), 'CommentBot', 8, 'commentbot', array(&$this, 'configWPSuperCommentBot'));
                        add_filter('plugin_action_links', array(&$this, 'filterPluginActions'), 10, 2);
                }

                /**
                 * adds settings link to plugin list
                 */
                function filterPluginActions($links, $file) {
                        static $this_plugin;
                        if (!$this_plugin) {
                                $this_plugin = plugin_basename(__FILE__);
                        }
                        if ($file == $this_plugin) {
                            	$settings_link = '<a href="plugins.php?page=commentbot">' . __('Settings', 'wpsupercommentbot') . '</a>';
                            	array_unshift($links, $settings_link); // before other links
                        }
                        return $links;
        		}

        		function showWpscbWarning() {
        		        echo '<div class="error"><p><strong>' . sprintf(__('Warning: CommentBot won\'t work until the cache folder (<code>%1$s</code>) is writeable (chmod 777).', 'wpsupercommentbot'), '/wp-content/plugins/' . basename(dirname(__FILE__)) . '/cache/') . '</strong></p></div>';
        		}

                /**
                 * checks cache folder and creates cache files
                 */
                function checkCacheFolder() {
                        $cacheFolder = $this->getCacheDir();
                        if (is_writeable($cacheFolder)) {
                                // create cache files
                                return $this->createCacheFiles();
                        }
                        else {
                                return false;
                        }
                }

                /**
                 * creates cache files
                 */
                function createCacheFiles() {
                        // get all source files
                        $incFolder = $this->getLocaleIncDir();
                        $incFiles = $this->getDirList($incFolder);
                        if (count($incFiles) > 0) {
                                foreach ($incFiles as $file) {
                                        $file = $incFolder . '/' . $file;
                                        if ($this->fileIsOutOfDate($file)) {
                                                $this->createPHPCacheFromTxt($file);
                                        }
                                }
                        }
                        return true;
                }

                /**
                 * checks if source file is newer then cache file
                 * @param string path to source file
                 */
                function fileIsOutOfDate($filename) {
                        $cacheFile = $this->getCacheDir() . '/' . basename($filename) . '.php';
                        if (!is_file($cacheFile)) {
                                return true;
                        }

                        // cache file is older than source file
                        if (@filemtime($filename) > @filemtime($cacheFile)) {
                                return true;
                        }
                        // cache file is up-to-date
                        else {
                                return false;
                        }
                }

                /**
                 * creates cache from txt file
                 * @param string file path
                 */
                function createPHPCacheFromTxt($filename) {
                        $incKey = basename($filename);
                        $isInteractionFile = ($incKey == 'comment_interaction.txt');
                        $lastKey = null;
                        $newLine = true;
                        $savedMultipleKeys = array();

                        $content  = "<?php\n\n/**\n * created @ " . date('d-m-Y H:i:s') . "\n * from " . $incKey . "\n */\n";
                        $incArrayKey = "\n" . '$inc["' . $incKey . '"]';

                        // read source file
                        $fd = @fopen($filename, "r");
                        if ($fd === false) {
                                return false;
                        }
                        while (!feof($fd)) {
                                $line = trim(fgets($fd, 4096));
                                if ($line != '') {
                                        if ($newLine === true) {
                                                $content .= $incArrayKey;
                                        }

                                        // is interaction file
                                        if ($isInteractionFile === true) {
                                                if (substr($line, 0, 1) == '!') {
                                                        $lastKey = substr($line, 1);
                                                        $newLine = false;
                                                        continue;
                                                }
                                                else {
                                                        // more then one key (key1|key2|key3)
                                                        if (strstr($lastKey, '|')) {
                                                                // multiple key doesn't exists yet
                                                                if (!isset($savedMultipleKeys[$lastKey])) {
                                                                        $keys = explode('|', $lastKey);
                                                                        $newKeys = array();
                                                                        foreach ($keys as $index => $key) {
                                                                                $key = trim($key);
                                                                                // first key -> extra wurst
                                                                                if ($index == 0) {
                                                                                        $newKeys[] = '["' . $key . '"]';
                                                                                }
                                                                                else {
                                                                                        $newKeys[] = '[] = $inc["' . $incKey . '"]["' . $key . '"]';
                                                                                }
                                                                        }
                                                                        $savedMultipleKeys[$lastKey] = implode('', $newKeys);
                                                                }
                                                                $content .= $savedMultipleKeys[$lastKey];
                                                        }
                                                        else {
                                                                $content .= '["' . $lastKey . '"]';
                                                        }
                                                        $newLine = true;
                                                }
                                        }
                                        $content .= '[] = "' . $this->prepareStringForCacheFile($line) . '";';
                                }
                        }
                        fclose($fd);
                        $content .= "\n" . '?>';

                        $newFilename = $this->getCacheDir() . '/' . $incKey . '.php';

                        // write cache content to cache file
                        $fw = fopen($newFilename, "wb");
                        $done = fwrite($fw, $content);
                        fclose($fw);

                        // no file created
                        if ($done === false) {
                                return false;
                        }

                        // set correct permissions
                        $stat = stat(dirname($newFilename));
                    	$perms = $stat['mode'] & 0000666;
                    	@chmod($newFilename, $perms);

                    	return true;
                }

                /**
                 * checks if a cache file exists
                 * @param string name of source file
                 */
                function getCacheFile($file) {
                        $done = true;
                        $filePath = $this->getLocaleIncDir() . '/' . $file;
                        // old or no cache file detected
                        if ($this->fileIsOutOfDate($filePath)) {
                                // create a new one
                                $done = $this->createPHPCacheFromTxt($filePath);
                        }

                        if ($done === false) {
                                return false;
                        }

                        // include cache file
                        include $this->getCacheDir() . '/' . $file . '.php';

                        // return cache content
                        return $inc[$file];
                }

                /**
                 * admin panel view
                 */
                function configWPSuperCommentBot() {
                        $wpscbOptions = get_option('WPSuperCommentBot');
                        $saved = false;

                        if (isset($_POST['submit'])) {
                                $validNonce = wp_verify_nonce($_REQUEST['_wpnonce'], 'wp-wpsupercommentbot-settings');
                                if ($validNonce === false) {
                                        die("Security problem!");
                                }

                                $validValues = array(
                                        'bot_name' => array('random', 'static'),
                                        'bot_activity' => array('often', 'medium', 'rare'),
                                        'bot_reaction' => array('positive', 'negative', 'random'),
                                        'bot_interaction' => array('yes', 'no'),
                                        'bot_url' => array('superlevel', 'none', 'else')
                                );
                                $standardValues = $this->standards;

                                // validate input
                                $validInputs = array();
                                foreach ($_POST['bot'] as $index => $value) {
                                        $validInputs[$index] = (is_array($validValues[$index]) && !in_array($value, $validValues[$index]) ? $standardValues[$index] : $value);
                                }

                                if ($validInputs['bot_name'] == 'static') {
                                        $botName = trim(stripslashes(strip_tags($_POST['bot']['bot_name_string'])));
                                        $validInputs['bot_name_string'] = ($botName == '' ? 'Superbot' : $botName);
                                }
                                else {
                                        $validInputs['bot_name_string'] = '';
                                }

                                if ($validInputs['bot_url'] == 'else') {
                                        $botUrl = trim(stripslashes(strip_tags($_POST['bot']['bot_url_string'])));
                                        $validInputs['bot_url_string'] = ($this->isUrl($botUrl) === false ? $this->url : $botUrl);
                                }
                                else {
                                        $validInputs['bot_url_string'] = '';
                                }

                                $botMail = trim(stripslashes(strip_tags($_POST['bot']['bot_mail_string'])));
                                $validInputs['bot_mail_string'] = ($this->isEmail($botMail) === false ? get_option('admin_email') : $botMail);

                                // new options
                                $wpscbOptions = array(
                                        'bot_id' => $wpscbOptions['bot_id'],
                                        'bot_name' => $validInputs['bot_name'],
                                        'bot_name_string' => $validInputs['bot_name_string'],
                                        'bot_activity' => $validInputs['bot_activity'],
                                        'bot_reaction' => $validInputs['bot_reaction'],
                                        'bot_interaction' => $validInputs['bot_interaction'],
                                        'bot_url' => $validInputs['bot_url'],
                                        'bot_url_string' => $validInputs['bot_url_string'],
                                        'bot_mail_string' => $validInputs['bot_mail_string']
                                );

                                update_option('WPSuperCommentBot', $wpscbOptions);
                				$saved = true;
                        }

                        if ($wpscbOptions['bot_mail_string'] == '') {
                                $wpscbOptions['bot_mail_string'] = get_option('admin_email');
                        }

                ?>

                        <div class="wrap">
				            <h2>WPSuperCommentBot <?php echo $this->version; ?> <?php _e('Configuration', 'wpsupercommentbot'); ?></h2>

                            <?php if ($saved === true) : ?>
				            <div class="updated fade below-h2"><p><?php _e('Changes has been saved.', 'wpsupercommentbot'); ?></p></div>
				            <?php endif; ?>

                            <p><em><?php _e('Note: If you don\'t want to use the bot anymore just deactivate the plugin. Your settings will be preserved.', 'wpsupercommentbot'); ?></em></p>

				            <br/><form action="" method="post">
				            <?php wp_nonce_field('wp-wpsupercommentbot-settings'); ?>

				            <h3><?php _e('General Settings', 'wpsupercommentbot'); ?></h3>
                            <table class="form-table">
                            <tr valign="top">
                                <th scope="row" style="width:25%"><?php _e('E-Mail', 'wpsupercommentbot'); ?></th>
                                <td><input name="bot[bot_mail_string]" class="regular-text" type="text" value="<?php echo $wpscbOptions['bot_mail_string']; ?>" /></td>
                            </tr>
                            <tr valign="top">
                                <th scope="row"><?php _e('Name', 'wpsupercommentbot'); ?></th>
                                <td>
                                    <input name="bot[bot_name]" type="radio" id="name_1" value="random"<?php $this->isChecked('bot_name', 'random', $wpscbOptions); ?> /> <label for="name_1"><?php _e('random name', 'wpsupercommentbot'); ?></label><br/>
                                    <input name="bot[bot_name]" type="radio" id="name_2" value="static"<?php $this->isChecked('bot_name', 'static', $wpscbOptions); ?> /> <label for="name_2"><?php _e('static name:', 'wpsupercommentbot'); ?></label> <input name="bot[bot_name_string]" type="text" class="regular-text" value="<?php echo $wpscbOptions['bot_name_string']; ?>" /><br/>
                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row"><?php _e('Website', 'wpsupercommentbot'); ?></th>
                                <td>
                                    <input name="bot[bot_url]" type="radio" id="url_2" value="none"<?php $this->isChecked('bot_url', 'none', $wpscbOptions); ?> /> <label for="url_2"><?php _e('none', 'wpsupercommentbot'); ?></label><br/>
                                    <input name="bot[bot_url]" type="radio" id="url_1" value="superlevel"<?php $this->isChecked('bot_url', 'superlevel', $wpscbOptions); ?> /> <label for="url_1">superlevel.de</label><br/>
                                    <input name="bot[bot_url]" type="radio" id="url_3" value="else"<?php $this->isChecked('bot_url', 'else', $wpscbOptions); ?> /> <label for="url_3"><?php _e('this one:', 'wpsupercommentbot'); ?></label> <input name="bot[bot_url_string]" type="text" class="regular-text" value="<?php echo $wpscbOptions['bot_url_string']; ?>" /><br/>
                                </td>
                            </tr>
                            </table>

                            <br/><h3><?php _e('Bot Finetuning', 'wpsupercommentbot'); ?></h3>
                            <table class="form-table">
                            <tr valign="top">
                                <th scope="row" style="width:25%"><?php _e('How active should the bot be?', 'wpsupercommentbot'); ?></th>
                                <td><input name="bot[bot_activity]" type="radio" id="activity_1" value="often"<?php $this->isChecked('bot_activity', 'often', $wpscbOptions); ?> /> <label for="activity_1"><?php _e('very active', 'wpsupercommentbot'); ?></label><br/>
                                    <input name="bot[bot_activity]" type="radio" id="activity_2" value="medium"<?php $this->isChecked('bot_activity', 'medium', $wpscbOptions); ?> /> <label for="activity_2"><?php _e('active', 'wpsupercommentbot'); ?></label><br/>
                                    <input name="bot[bot_activity]" type="radio" id="activity_3" value="rare"<?php $this->isChecked('bot_activity', 'rare', $wpscbOptions); ?> /> <label for="activity_3"><?php _e('rarely active', 'wpsupercommentbot'); ?></label><br/>
                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row"><?php _e('How should the bot respond to your articles?', 'wpsupercommentbot'); ?></th>
                                <td><input name="bot[bot_reaction]" type="radio" id="reaction_1" value="positive"<?php $this->isChecked('bot_reaction', 'positive', $wpscbOptions); ?> /> <label for="reaction_1"><?php _e('positive', 'wpsupercommentbot'); ?></label><br/>
                                    <input name="bot[bot_reaction]" type="radio" id="reaction_2" value="negative"<?php $this->isChecked('bot_reaction', 'negative', $wpscbOptions); ?> /> <label for="reaction_2"><?php _e('negative', 'wpsupercommentbot'); ?></label><br/>
                                    <input name="bot[bot_reaction]" type="radio" id="reaction_3" value="random"<?php $this->isChecked('bot_reaction', 'random', $wpscbOptions); ?> /> <label for="reaction_3"><?php _e('randomly', 'wpsupercommentbot'); ?></label><br/>
                                </td>
                            </tr>
                            <tr valign="top">
                                <th scope="row"><?php _e('Should the bot even respond to comments?', 'wpsupercommentbot'); ?></th>
                                <td><input name="bot[bot_interaction]" type="radio" id="interaction_1" value="yes"<?php $this->isChecked('bot_interaction', 'yes', $wpscbOptions); ?> /> <label for="interaction_1"><?php _e('yes', 'wpsupercommentbot'); ?></label><br/>
                                    <input name="bot[bot_interaction]" type="radio" id="interaction_2" value="no"<?php $this->isChecked('bot_interaction', 'no', $wpscbOptions); ?> /> <label for="interaction_2"><?php _e('no', 'wpsupercommentbot'); ?></label><br/>
                                </td>
                            </tr>
                            </table>

                            <p class="submit">
                                <br/><input name="submit" type="submit" class="button-primary" value="<?php _e('Save changes', 'wpsupercommentbot'); ?>" />
                            </p>
				            </form>
				        </div>

				        <?php
                }

                /**
                 * checks if the bot will comment and when (on post publish)
                 * @param int post id
                 * @param int internal counter
                 */
                function startWPSuperCommentBotOnPostSave($postID) {
                        if (get_post_status($postID) != 'publish' || $_POST['originalaction'] != 'post') {
                                return true;
                        }

                        $options = get_option('WPSuperCommentBot');
                        $counter = ceil($this->maxBotComments[$options['bot_activity']] * 0.5);

                        // number of saved bot comments
                        $botCommentCount = $this->getSavedBotComments($postID);

                        while ($counter > 0) {
                                // number of scheduled bot comments
                                $botScheduledCount = $this->getScheduledComments($postID);

                                // get max comments
                                $maxBotComments = $this->maxBotComments[$options['bot_activity']];

                                $botCount = ($botCommentCount + $botScheduledCount);

                                // no comment please
                                if ($botCount >= $maxBotComments) {
                                        return true;
                                }

                                $this->startBot($postID, $options);
                                $counter--;
                        }
                }

                function startBot($postID, $options) {
                        $chance = $this->calculateChance($options['bot_activity']);
                        // bot won't comment
                        if ($chance == 'n') {
                                return true;
                        }
                        // calculate comment timestamp
                        $timestamp = $this->calculateCommentTimestamp();
                        $unique = time() + rand(0, 90000);
                        wp_schedule_single_event($timestamp, 'wpscb_schedule', array($postID, $unique));
                }

                /**
                 * checks if and when the bot will comment (on comment post)
                 * @param int post id
                 */
                function startWPSuperCommentBotOnCommentSave($commentID) {
                        global $wpdb;

                        $options = get_option('WPSuperCommentBot');
                        // interaction is off
                        if ($options['bot_interaction'] == 'no') {
                                return true;
                        }

                        // how often did the bot already post
                        $comment = get_comment($commentID);
                        $postID = $comment->comment_post_ID;

                        // no interval on trackback and pingback
                        if (in_array($comment->comment_type, array('pingback', 'trackback'))) {
                                return true;
                        }

                        $sql = $wpdb->prepare(
                                "SELECT user_id
                                   FROM " . $wpdb->comments . "
                                  WHERE comment_ID = %d", $commentID
                        );
                        $commentUserID = $wpdb->get_var($sql);

                        // not approved or bot comment
                        if ($comment->comment_approved != '1' || $commentUserID == $options['bot_id']) {
                                return true;
                        }

                        // get max comments
                        $maxBotComments = $this->maxBotComments[$options['bot_activity']];

                        // number of saved bot comments
                        $botCommentCount = $this->getSavedBotComments($postID);

                        // number of scheduled bot comments
                        $botScheduledCount = $this->getScheduledComments($postID);

                        $botCount = ($botCommentCount + $botScheduledCount);

                        // no comment please
                        if ($botCount >= $maxBotComments) {
                                return true;
                        }

                        $chance = $this->calculateChance($options['bot_activity']);
                        // bot won't comment
                        if ($chance == 'n') {
                                return true;
                        }

                        // calculate comment timestamp
                        $timestamp = $this->calculateCommentOnCommentTimestamp();
                        // bot will comment on comment
                        wp_schedule_single_event($timestamp, 'wpscb_schedule_comment', array($postID, time()));
                }

                /**
                 * calculates the timestamp of auto comment
                 */
                function calculateCommentTimestamp() {
                        return time() + (rand($this->minMinutesFromNow, $this->maxMinutesFromNow) * 60);
                }

                /**
                 * calculates the timestamp of auto comment on comment
                 */
                function calculateCommentOnCommentTimestamp() {
                        return time() + (rand($this->minMinutesNextComment, $this->maxMinutesNextComment) * 60);
                }

                /**
                 * gets the name of the post author
                 * @param int post id
                 */
                function getPostAuthorName($postID) {
                        $post = get_post($postID);
                        $author = get_userdata($post->post_author);
                        $authorName = $author->display_name;
                        return $authorName;
                }

                /**
                 * saves the auto comment
                 * @param int post id
                 */
                function autoCommentOnPost($postID) {
                        // first of all: check if the post is even published
                        if (get_post_status($postID) != 'publish') {
                                return true;
                        }

                        $options = get_option('WPSuperCommentBot');

                        // getting author info
                        list($name, $email, $url) = $this->getCommentAuthorInfo($options);

                        // getting author of blog post
                        $authorName = $this->getPostAuthorName($postID);

                        // getting comment text
                        $text = $this->getCommentText($options, $authorName);

                        // error
                        if ($text === false) {
                                return true;
                        }

                        // all together
                        $commentData = $this->getCommentData($postID, $name, $email, $url, $text);
                        $newCommentId = wp_new_comment($commentData);
                }

                /**
                 * saves the auto comment
                 * @param int comment id
                 */
                function autoCommentOnComment($postID) {
                        global $wpdb;
                        $options = get_option('WPSuperCommentBot');

                        // approved comments, no pingbacks, no trackbacks
                        $sqlWhere = $wpdb->prepare("comment_post_ID = %d AND comment_approved = '1' AND comment_type != 'pingback' AND comment_type != 'trackback'", $postID);
                        // no soliloquy
                        if ($options['bot_name'] != 'random') {
                                $sqlWhere .= $wpdb->prepare(" AND comment_author != %s", $options['bot_name_string']);
                        }

                        // get all comments
                        $comments = $wpdb->get_results("SELECT comment_ID, comment_content, comment_author FROM $wpdb->comments WHERE " . $sqlWhere);
                        if (!$comments) {
                                return true;
                        }

                        $max = count($comments) - 1;
                        // random comment
                        $comment = $comments[rand(0, $max)];

                        // getting author info
                        list($name, $email, $url) = $this->getCommentAuthorInfo($options);

                        // remove old blockquotes
                        $content = $this->removeOldBlockquotes($comment->comment_content);

                        $newComment = $this->getCommentText($options, '', $content);

                        // error
                        if ($newComment === false) {
                                return true;
                        }

                        $commentLink = get_comment_link($comment->comment_ID);
                        // wrong comment link > break
                        if (!$commentLink || !$content) {
                                return true;
                        }

                        // getting comment text
                        $text  = '<a href="' . $commentLink . '">@' . $comment->comment_author . ':</a>' . "\n\n";
                        $text .= '<blockquote>' . $content . '</blockquote>';
                        $text .= "\n\n" . $newComment;

                        // all together
                        $commentData = $this->getCommentData($postID, $name, $email, $url, $text, $comment->comment_ID);
                        $newCommentId = wp_new_comment($commentData);
                }

                /**
                 * returns the comment data
                 * @param int    post id
                 * @param string name of comment author
                 * @param string email of comment author
                 * @param string website of comment author
                 * @param string comment content
                 * @param int    comment parent
                 */
                function getCommentData($postID, $name, $email, $url, $text, $commentParentID = 0) {
                        $options = get_option('WPSuperCommentBot');
                        return array(
                                'comment_post_ID' => $postID,
                                'comment_content' => $text,
                                'comment_author' => $name,
                                'comment_author_url' => $url,
                                'comment_author_email' => $email,
                                'comment_parent' => $commentParentID,
                                'user_ID' => $options['bot_id']
                        );
                }

                function getScheduledComments($postID) {
                        $cron = _get_cron_array();
                        $cnt = 0;
                        foreach ($cron as $index => $c) {
                                foreach ($c as $key => $array) {
                                        if (in_array($key, array('wpscb_schedule', 'wpscb_schedule_comment'))) {
                                                foreach ($array as $bla) {
                                                        if ($postID == $bla['args'][0]) {
                                                                 $cnt++;
                                                        }
                                                }
                                        }
                                }
                        }
                        return $cnt;
                }

                function getSavedBotComments($postID) {
                        global $wpdb;
                        $options = get_option('WPSuperCommentBot');
                        $botID = $options['bot_id'];
                        $commentCount = $wpdb->get_var(
                                "SELECT COUNT(comment_ID)
                                   FROM $wpdb->comments
                                  WHERE user_id = " . $botID . "
                                    AND comment_approved = '1'
                                    AND comment_post_id = " . $postID
                        );
                        return absint($commentCount);
                }

                /**
                 * gets author info based on the saved options
                 * @param array options
                 */
                function getCommentAuthorInfo($options) {
                        $email = $options['bot_mail_string'];
                        $name = ($options['bot_name'] == 'random') ? $this->getRandomItem('names.txt') : $options['bot_name_string'];
                        $url = '';
                        if ($options['bot_url'] != 'none') {
                                $url = ($options['bot_url'] == 'superlevel') ? $this->url : $options['bot_url_string'];
                        }
                        return array($name, $email, $url);
                }

                /**
                 * gets path to inc directory
                 */
                function getLocaleIncDir() {
                        $lang = get_locale();
                        $dir = dirname(__FILE__) . '/inc/' . $lang;
                        if (!is_dir($dir)) {
                                $dir = dirname(__FILE__) . '/inc/' . $this->standardLanguage;
                        }
                        return $dir;
                }

                /**
                 * gets comment text based on the saved options
                 * @param array  options
                 * @param string (optional) name of post author
                 * @param string (optional) content of quoted comment
                 */
                function getCommentText($options, $authorName, $commentContent = '') {
                        $reaction = $options['bot_reaction'];

                        // comment on comment
                        if ($commentContent != '') {
                                $phraseFile = 'comment_phrases.txt';
                        }
                        // comment on post
                        else {
                                $wordLists = array('positive', 'negative');
                                $file = ($reaction == 'random') ? $wordLists[rand(0, 1)] : $reaction;
                                $phraseFile = $file . '_phrases.txt';
                        }

                        if ($commentContent != '') {
                                return $this->getInteractionText($commentContent);
                        }

                        // return just a predefined phrase (50% chance)
                        if ($this->calculateChance('medium') == 'y') {
                                return $this->getRandomItem($phraseFile);
                        }

                        // build text
                        return $this->getRandomTextFromFile($file, $authorName);
                }

                /**
                 * gets a random text from the txt files
                 * @param string kind of file (negative or positive)
                 * @param string (optional) name of post author
                 */
                function getRandomTextFromFile($kindOfFile, $authorName = '') {
                        $files = array('prefixes', 'adjectives', 'adjectives', 'saluts');
                        $texts = array();
                        $noPrefix = false;
                        foreach ($files as $index => $file) {
                                $temp = '';

                                // 50% chance for prefix
                                if ($index == 0) {
                                        if ($this->calculateChance('medium') == 'y') {
                                                $temp = $this->getRandomItem($kindOfFile . '_' . $file . '.txt', $authorName);
                                        }
                                        else {
                                                $noPrefix = true;
                                        }
                                }
                                // 50% chance for second adjective
                                else if ($index == 2) {
                                        $temp = $texts[$file];
                                        if ($this->calculateChance('medium') == 'y') {
                                                // add second adjective and glue it to the first with "and"
                                                $temp .= ' ' . __('and', 'wpsupercommentbot') . ' ' . $this->getRandomItem($kindOfFile . '_' . $file . '.txt', $authorName);
                                        }
                                }
                                // 50% chance for salut
                                else if ($index == 3) {
                                        if ($this->calculateChance('medium') == 'y') {
                                                $temp = $this->getRandomItem($kindOfFile . '_' . $file . '.txt', $authorName);
                                        }
                                }
                                else {
                                        $temp = $this->getRandomItem($kindOfFile . '_' . $file . '.txt', $authorName);
                                }

                                // break on error
                                if ($temp === false) {
                                        return false;
                                }

                                // save text
                                $texts[$file] = $temp;
                        }

                        $booster = $this->getRandomItem('booster.txt');
                        $extra = $this->getRandomItem('extra.txt');

                        // break on error
                        if ($extra === false || $booster === false) {
                                return false;
                        }
                        // extra phrase
                        $texts['extra'] = $extra;

                        if ($noPrefix === true) {
                                // no prefix means camelize the extra
                                $texts['extra'] = $this->camelizeFirstWordOfString($texts['extra']);
                                // and calculate if author name or not (25%)
                                if ($this->calculateChance('rare') == 'y') {
                                        $texts['adjectives'] .= ', ' . $authorName;
                                }
                        }

                        $result = '';
                        if ($noPrefix === false) {
                                $result .= $texts['prefixes'] . ', ';
                        }
                        $result .= $texts['extra'] . ' ';
                        // 50% chance
                        if ($this->calculateChance('medium') == 'y') {
                                $result .= $booster . ' ';
                        }
                        $result .= $texts['adjectives'] . '. ';
                        $result .= $texts['saluts'];

                        return $result;
                }

                /**
                 * parses content and gets matching reactions
                 * @param string content of comment
                 */
                function getInteractionText($content) {
                        $dir = $this->getLocaleIncDir();

                        // get all inc filenames, exclude comment_interaction.txt
                        $incFiles = $this->getDirList($dir, 'comment_interaction.txt');

                        $lines = array();
                        $lastKey = '';

                        $lines = $this->getCacheFile('comment_interaction.txt');

                        // break on error
                        if ($lines === false) {
                                return false;
                        }

                        // content clean up
                        $content = ' ' . $this->cleanUpText($content) . ' ';

                        // if a key exists
                        $keys = strtolower(implode('|', array_keys($lines)));
                        $keys = str_replace('*', '[^\s]*', $keys);
                        if (preg_match('=[\s]+(' . $keys . ')[\s]+=i', $content, $matches)) {
                                $key = strtolower(trim($matches[0]));
                                // wildcard support
                                if (!isset($lines[$key])) {
                                        $key .= '*';
                                }
                                if (!isset($lines[$key])) {
                                        $key = '*' . substr($key, 0, -1);
                                }
                                // return respond
                                $randomItem = rand(0, (count($lines[$key]) - 1));
                                $return = $lines[$key][$randomItem];

                                // random from inc file
                                $return1 = $return;
                                if (isset($incFiles[$return1])) {
                                        $return = $this->getRandomItem($return1);
                                }
                		}
                        else {
                                // return random phrase
                                $return = $this->getRandomItem('comment_phrases.txt');
                        }
                        return $return;
                }

                /**
                 * gets a random item from one of the txt files
                 * @param string filename
                 * @param string (optional) name of post author
                 */
                function getRandomItem($filename, $authorName = '') {
                        $lines = $this->getCacheFile($filename);

                        // break on error
                        if ($lines === false) {
                                return false;
                        }
                        // add post author name to prefixes
                        if (strstr($filename, 'prefixes.txt') && $authorName != '') {
                                $lines[] = $authorName;
                        }

                        // get random line
                        return $lines[rand(0, (count($lines)-1))];
                }

        }

}

$WPSuperCommentBot = new WPSuperCommentBot();
